@@ -2691,7 +2691,7 @@ parse_done:
 		sb.commits.compare = compare_commits_by_commit_date;
 	}
 	else if (contents_from)
-		die("--contents and --children do not blend well.");
+		die("--contents and --reverse do not blend well.");
 	else if (revs.first_parent_only)
 		die("combining --first-parent and --reverse is not supported");
 	else {
